import { CPCWallet, BigNumber, CPCJsonRpcProvider, Contract, TxResult, uint256, bool, address, uint64, uint8, int8 } from 'cpchain-typescript-sdk';
export interface HandGameEntity {
    gameId: uint64;
    amount: uint256;
    starter: address;
    player: address;
    status: uint8;
    starterProof: uint256;
    playerProof: uint256;
    timeout: uint256;
    threshold: uint256;
}
export interface ContentVerified {
    proof: uint256;
    key: uint256;
    content: uint256;
}
export interface EnableStatusChanged {
    enabled: bool;
}
export interface OwnershipTransferred {
    previousOwner: address;
    newOwner: address;
}
export interface SetMaxLimit {
    limit: uint256;
}
export interface SetTimeoutLimit {
    limit: uint256;
}
export interface GameStarted {
    gameId: uint64;
    starter: address;
    card: uint256;
    amount: uint256;
    threshold: uint256;
}
export interface CreateGroupHandGame {
    group_id: uint256;
    gameId: uint64;
    starter: address;
    message: string;
    amount: uint256;
    threshold: uint256;
}
export interface GameCancelled {
    gameId: uint64;
}
export interface GameLocked {
    gameId: uint64;
    player: address;
    card: uint256;
    amount: uint256;
}
export interface CardOpened {
    gameId: uint64;
    player: address;
    key: uint256;
    content: uint256;
}
export interface GameFinished {
    gameId: uint64;
    result: int8;
}
export declare class GameViewer {
    provider: CPCJsonRpcProvider;
    contractAddress: address;
    contract: Contract;
    constructor(provider: CPCJsonRpcProvider, contractAddress: address);
    games(i: uint256): Promise<[uint64, uint256, address, address, uint8, uint256, uint256, uint256, uint256]>;
    maxLimit(): Promise<[uint256]>;
    gameToGroup(i: uint64): Promise<[uint256]>;
    validateProof(proof: uint256, key: uint256, content: uint256): Promise<[bool]>;
    enabled(): Promise<[bool]>;
    timeoutLimit(): Promise<[uint256]>;
    groupChatAddress(): Promise<[address]>;
    totalGameNumber(): Promise<[uint64]>;
    viewContent(proof: uint256): Promise<[uint256, uint256]>;
    owner(): Promise<[address]>;
    pendingOwner(): Promise<[address]>;
    viewGame(gameId: uint64): Promise<[uint256, uint256, address, address, uint8, uint256, uint256[]]>;
    viewLatestGames(limit: uint64): Promise<[uint256[]]>;
    balanceOf(account: address): Promise<[uint256]>;
}
export declare class GameCaller {
    wallet: CPCWallet;
    contractAddress: address;
    contract: Contract;
    constructor(wallet: CPCWallet, contractAddress: address);
    enableContract(_tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    setProof(proof: uint256, key: uint256, content: uint256, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    claimOwnership(_tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    renounceOwnership(_tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    disableContract(_tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    transferOwnership(newOwner: address, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    setGroupChat(groupchat: address, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    setMaxLimit(limit: uint256, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    setTimeoutLimit(limit: uint256, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    startGame(proof: uint256, threshold: uint256, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    startGroupChatGame(proof: uint256, threshold: uint256, group_id: uint256, userMessage: string, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    cancelGame(gameId: uint64, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    joinGame(gameId: uint64, proof: uint256, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    openCard(gameId: uint64, key: uint256, content: uint256, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
    finishTimeoutGame(gameId: uint64, _tx?: {
        value?: BigNumber;
    }): Promise<TxResult>;
}
